#ifndef SIMVERIFIX_H
#define SIMVERIFIX_H

#include <system/system.h>
#include <system/task.h>
#include <search/successor_generator.h>
#include <search/normalizer.h>
#include <unordered_map>
#include <list>
#include <vector>

using namespace std;

/** A hash table for discrete parts of states */
class DiscretePartTable
{
public:
  ~DiscretePartTable();
  /** Checks if a state equal to *dp is already there.
	  If yes, *destroys* dp and returns the pointer to the existing object.
	  Otherwise adds dp to the hash table and returns it.
  */
  DiscretePart* addDiscretePart(DiscretePart * dp);

  inline int keySize() const {
	return hashtbl.size();
  }
  //  unordered_map<DiscretePart *, list<DiscretePart*>, DiscretePartHash> hashtbl;
  unordered_map<uint32_t, list<DiscretePart*>> hashtbl;
};

class TaskBundle{
 public: 
 TaskBundle(Task * task) : task(task){
	succgen = new SuccessorGenerator(task->system);
	norm = new LocalLUNormalizer(task->system);
  }
  ~TaskBundle(){
	delete(task);
	delete(succgen);
	delete(norm);
  }
  inline System * getSystem() const{
	return task->system;
  }
  inline Target * getTarget() const{
	return task->target;
  }
  /*
  inline getLUBounds(vector<int32_t> & floors, vector<int32_t>&ceils, DiscretePart * dp)
  {
	norm->getLUBounds(floors,ceils,dp);
  }
  */
  Task * task;
  SuccessorGenerator * succgen;
  DiscretePartTable table;
  LocalLUNormalizer * norm;
};


enum dbm_inequality_type  { DBM_WEAK, DBM_STRICT };
enum extended_int { Infty, Int };


TaskBundle * readTask(const char * ta, const char * query);
void closeTask(TaskBundle * t);
int getNrClocks(TaskBundle * t);
int getDimension(TaskBundle * t);
DiscretePart * getInitialState(TaskBundle * t);
void getTransitions(TaskBundle * t, DiscretePart * dp,  std::vector<const Transition*>& succs);
DiscretePart * applyTransition(TaskBundle * t, DiscretePart * dp, const Transition * trans);
bool checkTask(TaskBundle * tb);
int * guardOfTransition(TaskBundle * tb, const Transition * trans);
int * invariantOfDiscretePart(TaskBundle * tb, const DiscretePart * dp);
vector<cindex_t> resetOfTransition(TaskBundle *tb, const Transition * trans);
void printBoundMatrix(int ** bounds, int dim);
char * stringOfDiscretePart(TaskBundle * tb, const DiscretePart * dp);
char * stringOfTransition(const Transition * t);
bool isTarget(TaskBundle * tb, DiscretePart * dp);
void getClockNames(TaskBundle * tb, vector< pair<unsigned,string> > & clock_names);
void getLUBounds(TaskBundle * tb, vector<int32_t> & floors, vector<int32_t> & ceils, DiscretePart * dp);
bool isUrgentOrCommitted(TaskBundle*,DiscretePart*);

static inline
int32_t raw2bound(raw_t raw)
{
    return (raw >> 1);
}

static inline
strictness_t raw2strict(raw_t raw)
{
    return (strictness_t)(raw & 1);
}
inline int getMinusInfinity()
{
  return -dbm_INFINITY;
}
inline int getInfinity()
{
  return dbm_INFINITY;
}



#endif
