extern "C" {
#include <stdio.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/callback.h>
}
#include <dbm/constraints.h>
#include <simverifix.h>
#include <system/task.h>
#include <vector>

extern "C" CAMLprim value vx_readTask(value ta, value query)
{
  CAMLparam2(ta,query);
	char * sta = String_val(ta);
	char * squery = String_val(query);
  TaskBundle * t = readTask(sta, squery);
  CAMLreturn(value(t));
}

extern "C" CAMLprim value vx_getNrClocks(value t)
{
	CAMLparam1(t);
	TaskBundle * task = (TaskBundle*) t;
	CAMLreturn(Val_int(getNrClocks(task)));
}
extern "C" CAMLprim value vx_getDimension(value t)
{
	CAMLparam1(t);
	TaskBundle * task = (TaskBundle*) t;
	CAMLreturn(Val_int(getDimension(task)));
}

extern "C" void vx_closeTask(value tb)
{
  CAMLparam1(tb);
  closeTask((TaskBundle*)tb);
  CAMLreturn0;
}

extern "C" CAMLprim value vx_getInitialState(value ptb)
{
  CAMLparam1(ptb);
  TaskBundle * tb = (TaskBundle*) ptb;
  CAMLreturn(value(getInitialState(tb)));
}

extern "C" CAMLprim value vx_getTransitions(value ptb, value pdp)
{
  CAMLparam2(ptb,pdp);
  CAMLlocal2(cli, cons);
  TaskBundle * tb = (TaskBundle*) ptb;
  DiscretePart * dp = (DiscretePart*) pdp;
  std::vector<const Transition *> succs;
  getTransitions(tb,dp,succs);
  cli = Val_emptylist;
  for(int i = 0; i < succs.size(); i++){
	cons = caml_alloc(2, 0);
	Store_field( cons, 0, value(succs[i]) );  // head
    Store_field( cons, 1, cli );              // tail
    cli = cons;	
  }
  CAMLreturn(cli);
}
extern "C" CAMLprim value vx_applyTransition(value ptb, value pdp, value ptrans)
{
  CAMLparam3(ptb,pdp,ptrans);
  TaskBundle * tb = (TaskBundle*) ptb;
  DiscretePart * dp = (DiscretePart*) pdp;
  Transition * trans = (Transition*) ptrans;
  DiscretePart * target = applyTransition(tb,dp,trans);
  CAMLreturn((value)target);
}

extern "C" CAMLprim value vx_stringOfTransition(value ptrans)
{
  CAMLparam1(ptrans);
  char * cstr = stringOfTransition((Transition*)ptrans);
  // TODO I think I need to free cstr here
  CAMLreturn(caml_copy_string(cstr));
}
extern "C" CAMLprim value vx_stringOfDiscretePart(value ptb, value pdp)
{
  CAMLparam2(ptb, pdp);
  TaskBundle * tb = (TaskBundle*)ptb;
  DiscretePart * dp  = (DiscretePart*)pdp;
  char * cstr = stringOfDiscretePart(tb, dp);
  // TODO I think I need to free cstr here
  CAMLreturn(caml_copy_string(cstr));
}

extern "C" CAMLprim value vx_resetOfTransition(value ptb, value ptr)
{
  CAMLparam2(ptb,ptr);
  CAMLlocal2(cli, cons);
  TaskBundle * tb = (TaskBundle*)ptb;
  Transition * tr = (Transition*)ptr;
  std::vector<unsigned> rv = resetOfTransition(tb,tr);
  cli = Val_emptylist;
  for(int i = 0; i < rv.size(); i++){
	cons = caml_alloc(2, 0);
	Store_field( cons, 0, Val_int(rv[i]) );  // head
    Store_field( cons, 1, cli );              // tail
    cli = cons;	
  }
  CAMLreturn(cli);
}

extern "C" CAMLprim value vx_getClockNames(value ptb)
{
  CAMLparam1(ptb);
  CAMLlocal3(cli, cons, newpair);
  TaskBundle * tb = (TaskBundle*)ptb;
  vector<pair<unsigned,string> > clock_names;
  getClockNames(tb,clock_names);
  cli = Val_emptylist;
  for(int i = 0; i < clock_names.size(); i++){
	int clock_id = (int) clock_names[i].first;
	string clock_name = clock_names[i].second;
	newpair = caml_alloc(2,0);
	Store_field(newpair, 0, Val_int(clock_id));
	Store_field(newpair, 1, caml_copy_string(clock_name.c_str()));
	cons = caml_alloc(2, 0);
	Store_field( cons, 0, newpair );  // head
    Store_field( cons, 1, cli );              // tail
    cli = cons;	
  }
  CAMLreturn(cli);
}

/* Reminder:
enum dbm_inequality_type  { DBM_WEAK, DBM_STRICT };
enum extended_int { Infty, Int };
*/

CAMLprim value makeGuard(int * bounds, int dim){
  CAMLlocal4(g,pret, tuple, extint);
  const int infty = getInfinity();
  //  cout << "Infty: " << infty << endl;

  g = caml_alloc(dim*dim,0);
  for (int i = 0; i < dim; i++){
	for(int j = 0; j < dim; j++){
	  tuple = caml_alloc(2,0); // pair of (extint,ineq)
	  if ( raw2bound(bounds[i * dim + j]) == infty ) {		
		Store_field(tuple, 0, Val_int(0)); // Infty
		Store_field(tuple, 1, Val_int(1)); // DBM_STRICT
	  } else {
		int ineq = !raw2strict(bounds[i * dim + j]);
		int bound = raw2bound(bounds[i * dim + j]);
		extint = caml_alloc(1,1); // allocate one value with tag 1
		Store_field(extint, 0, Val_int(bound));
		Store_field(tuple, 0, extint);
		Store_field(tuple, 1, Val_int(ineq));
	  }
	  Store_field(g, i*dim + j, tuple);
	}
  }
  pret = caml_alloc(2,0);
  Store_field(pret, 0, Val_int(dbm_INFINITY));
  Store_field(pret, 1, g);
  return g;
}


extern "C" CAMLprim value vx_invariantOfDiscreteState(value ptb, value pstate){
  CAMLparam2(ptb,pstate);
  CAMLlocal2(g,pret);  
  TaskBundle * tb = (TaskBundle*)ptb;
  int dim = getDimension(tb);
  DiscretePart * dp = (DiscretePart*)pstate;
  int * bounds = invariantOfDiscretePart(tb,dp);
  CAMLprim value ret = makeGuard(bounds, dim);
  delete(bounds);
  CAMLreturn(ret);
}


extern "C" CAMLprim value vx_guardOfTransition(value ptb, value ptr){
  CAMLparam2(ptb,ptr);
  TaskBundle * tb = (TaskBundle*)ptb;
  const int dim = getDimension(tb);
  Transition * tr = (Transition*)ptr;
  int * bounds = guardOfTransition(tb,tr);
  CAMLprim value ret = makeGuard(bounds, dim);
  delete(bounds);
  CAMLreturn(ret);
}

extern "C" CAMLprim value vx_isTarget(value ptb, value pdp){
    CAMLparam2(ptb,pdp);
	CAMLreturn(Val_bool(isTarget((TaskBundle*)ptb, (DiscretePart*)pdp)));
}

extern "C" CAMLprim value vx_getLUBounds(value ptb, value pdp, value lbounds, value ubounds){
  CAMLparam4(ptb, pdp, lbounds, ubounds);
  static vector<int32_t> floors, ceils;
  TaskBundle * tb = (TaskBundle*) ptb;
  DiscretePart * dp = (DiscretePart*) pdp;
  getLUBounds(tb, floors, ceils, dp);
  int dim = getDimension(tb);
  int minfty = getMinusInfinity();
  for (int i = 0; i < dim; i++){
	Store_field(lbounds, i, Val_int(floors[i]));
	Store_field(ubounds, i, Val_int(ceils[i]));
  }
  CAMLreturn(Val_unit);
}
extern "C" CAMLprim value vx_getScaledLUBounds(value ptb, value pdp, value cscale, value cenlarge, value lbounds, value ubounds){
  CAMLparam4(ptb, pdp, lbounds, ubounds);
  CAMLxparam2(cscale, cenlarge);
  static vector<int32_t> floors, ceils;
  int scale = Int_val(cscale);
  int enlarge = Int_val(cenlarge);
  TaskBundle * tb = (TaskBundle*) ptb;
  DiscretePart * dp = (DiscretePart*) pdp;
  getLUBounds(tb, floors, ceils, dp);
  int dim = getDimension(tb);
  int minfty = getMinusInfinity();
  for (int i = 0; i < dim; i++){
	Store_field(lbounds, i, Val_int(floors[i]));
	Store_field(ubounds, i, Val_int(ceils[i]));
  }
  CAMLreturn(Val_unit);
}

extern "C" CAMLprim value vx_getInfinity(){
  return Val_int(getInfinity());
}
extern "C" CAMLprim value vx_getMinusInfinity(){
  return Val_int(getMinusInfinity());
}
extern "C" CAMLprim value vx_isUrgentOrCommitted(value ptb, value pdp){
  CAMLparam2(ptb,pdp);
  TaskBundle * tb = (TaskBundle*) ptb;
  DiscretePart * dp = (DiscretePart*) pdp;
  CAMLreturn(Val_bool(isUrgentOrCommitted(tb,dp)));
}
